@@ -1,4 +1,4 @@
-import os
+"""Test the pypdf._merger module."""
 import sys
 from io import BytesIO
 from pathlib import Path
@@ -7,10 +7,9 @@ import pytest
 
 import pypdf
 from pypdf import PdfMerger, PdfReader, PdfWriter
-from pypdf.errors import DeprecationError
 from pypdf.generic import Destination, Fit
 
-from . import get_pdf_from_url
+from . import get_data_from_url
 
 TESTS_ROOT = Path(__file__).parent.resolve()
 PROJECT_ROOT = TESTS_ROOT.parent
@@ -19,6 +18,7 @@ RESOURCE_ROOT = PROJECT_ROOT / "resources"
 sys.path.append(str(PROJECT_ROOT))
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def merger_operate(merger):
     pdf_path = RESOURCE_ROOT / "crazyones.pdf"
     outline = RESOURCE_ROOT / "pdflatex-outline.pdf"
@@ -130,7 +130,7 @@ def merger_operate(merger):
     merger.add_metadata({"/Author": "Martin Thoma"})
     merger.add_named_destination("/Title", 0)
     merger.set_page_layout("/SinglePage")
-    merger.set_page_mode("/UseThumbs")
+    merger.page_mode = "/UseThumbs"
 
 
 def check_outline(tmp_path):
@@ -156,6 +156,7 @@ def check_outline(tmp_path):
 tmp_filename = "dont_commit_merged.pdf"
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_merger_operations_by_traditional_usage(tmp_path):
     # Arrange
     merger = PdfMerger()
@@ -183,6 +184,7 @@ def test_merger_operations_by_traditional_usage_with_writer(tmp_path):
     check_outline(path)
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_merger_operations_by_semi_traditional_usage(tmp_path):
     path = tmp_path / tmp_filename
 
@@ -191,7 +193,7 @@ def test_merger_operations_by_semi_traditional_usage(tmp_path):
         merger.write(path)  # Act
 
     # Assert
-    assert os.path.isfile(path)
+    assert Path(path).is_file()
     check_outline(path)
 
 
@@ -203,16 +205,17 @@ def test_merger_operations_by_semi_traditional_usage_with_writer(tmp_path):
         merger.write(path)  # Act
 
     # Assert
-    assert os.path.isfile(path)
+    assert Path(path).is_file()
     check_outline(path)
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_merger_operation_by_new_usage(tmp_path):
     path = tmp_path / tmp_filename
     with PdfMerger(fileobj=path) as merger:
         merger_operate(merger)
     # Assert
-    assert os.path.isfile(path)
+    assert Path(path).is_file()
     check_outline(path)
 
 
@@ -222,10 +225,11 @@ def test_merger_operation_by_new_usage_with_writer(tmp_path):
         merger_operate(merger)
 
     # Assert
-    assert os.path.isfile(path)
+    assert Path(path).is_file()
     check_outline(path)
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_merge_page_exception():
     merger = pypdf.PdfMerger()
     pdf_path = RESOURCE_ROOT / "crazyones.pdf"
@@ -247,6 +251,7 @@ def test_merge_page_exception_with_writer():
     merger.close()
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_merge_page_tuple():
     merger = pypdf.PdfMerger()
     pdf_path = RESOURCE_ROOT / "crazyones.pdf"
@@ -261,6 +266,7 @@ def test_merge_page_tuple_with_writer():
     merger.close()
 
 
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_merge_write_closed_fh():
     merger = pypdf.PdfMerger()
     pdf_path = RESOURCE_ROOT / "crazyones.pdf"
@@ -270,7 +276,7 @@ def test_merge_write_closed_fh():
 
     merger.close()
     with pytest.raises(RuntimeError) as exc:
-        merger.write("stream.pdf")
+        merger.write("test_merge_write_closed_fh.pdf")
     assert exc.value.args[0] == err_closed
 
     with pytest.raises(RuntimeError) as exc:
@@ -282,7 +288,7 @@ def test_merge_write_closed_fh():
     assert exc.value.args[0] == err_closed
 
     with pytest.raises(RuntimeError) as exc:
-        merger.set_page_mode("/UseNone")
+        merger.page_mode = "/UseNone"
     assert exc.value.args[0] == err_closed
 
     with pytest.raises(RuntimeError) as exc:
@@ -298,234 +304,197 @@ def test_merge_write_closed_fh():
     assert exc.value.args[0] == err_closed
 
 
-def test_merge_write_closed_fh_with_writer():
+def test_merge_write_closed_fh_with_writer(pdf_file_path):
     merger = pypdf.PdfWriter()
     pdf_path = RESOURCE_ROOT / "crazyones.pdf"
     merger.append(pdf_path)
 
     merger.close()
-    merger.write("stream1.pdf")
+    merger.write(pdf_file_path)
     merger.add_metadata({"author": "Martin Thoma"})
     merger.set_page_layout("/SinglePage")
-    merger.set_page_mode("/UseNone")
+    merger.page_mode = "/UseNone"
     merger.add_outline_item("An outline item", 0)
 
-    os.unlink("stream1.pdf")
 
-
-@pytest.mark.external
-def test_trim_outline_list():
+@pytest.mark.enable_socket()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_trim_outline_list(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/995/995175.pdf"
     name = "tika-995175.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-def test_trim_outline_list_with_writer():
+@pytest.mark.enable_socket()
+def test_trim_outline_list_with_writer(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/995/995175.pdf"
     name = "tika-995175.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-def test_zoom():
+@pytest.mark.enable_socket()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_zoom(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/994/994759.pdf"
     name = "tika-994759.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-def test_zoom_with_writer():
+@pytest.mark.enable_socket()
+def test_zoom_with_writer(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/994/994759.pdf"
     name = "tika-994759.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-def test_zoom_xyz_no_left():
+@pytest.mark.enable_socket()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_zoom_xyz_no_left(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/933/933322.pdf"
     name = "tika-933322.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-def test_zoom_xyz_no_left_with_writer():
+@pytest.mark.enable_socket()
+def test_zoom_xyz_no_left_with_writer(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/933/933322.pdf"
     name = "tika-933322.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-def test_outline_item():
+@pytest.mark.enable_socket()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_outline_item(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/997/997511.pdf"
     name = "tika-997511.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
-def test_outline_item_with_writer():
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
+def test_outline_item_with_writer(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/997/997511.pdf"
     name = "tika-997511.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
-def test_trim_outline():
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_trim_outline(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/982/982336.pdf"
     name = "tika-982336.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
-def test_trim_outline_with_writer():
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
+def test_trim_outline_with_writer(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/982/982336.pdf"
     name = "tika-982336.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
-def test1():
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test1(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/923/923621.pdf"
     name = "tika-923621.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
-def test1_with_writer():
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
+def test1_with_writer(pdf_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/923/923621.pdf"
     name = "tika-923621.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
-def test_sweep_recursion1():
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_sweep_recursion1(pdf_file_path):
     # TODO: This test looks like an infinite loop.
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924546.pdf"
     name = "tika-924546.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    reader2 = PdfReader("tmp-merger-do-not-commit.pdf")
+    reader2 = PdfReader(pdf_file_path)
     reader2.pages
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
-def test_sweep_recursion1_with_writer():
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
+def test_sweep_recursion1_with_writer(pdf_file_path):
     # TODO: This test looks like an infinite loop.
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924546.pdf"
     name = "tika-924546.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    reader2 = PdfReader("tmp-merger-do-not-commit.pdf")
+    reader2 = PdfReader(pdf_file_path)
     reader2.pages
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
 @pytest.mark.parametrize(
     ("url", "name"),
     [
@@ -540,22 +509,20 @@ def test_sweep_recursion1_with_writer():
         ),
     ],
 )
-def test_sweep_recursion2(url, name):
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_sweep_recursion2(url, name, pdf_file_path):
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfMerger()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
     merger.close()
 
-    reader2 = PdfReader("tmp-merger-do-not-commit.pdf")
+    reader2 = PdfReader(pdf_file_path)
     reader2.pages
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
 
-
-@pytest.mark.external
-@pytest.mark.slow
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
 @pytest.mark.parametrize(
     ("url", "name"),
     [
@@ -570,122 +537,76 @@ def test_sweep_recursion2(url, name):
         ),
     ],
 )
-def test_sweep_recursion2_with_writer(url, name):
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
-    merger = PdfMerger()
-    merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
-    merger.close()
-
-    reader2 = PdfReader("tmp-merger-do-not-commit.pdf")
-    reader2.pages
-
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
-
-
-@pytest.mark.external
-def test_sweep_indirect_list_newobj_is_none(caplog):
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/906/906769.pdf"
-    name = "tika-906769.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
-    merger = PdfMerger()
-    merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
-    merger.close()
-    # used to be: assert "Object 21 0 not defined." in caplog.text
-
-    reader2 = PdfReader("tmp-merger-do-not-commit.pdf")
-    reader2.pages
-
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
-
-
-@pytest.mark.external
-def test_sweep_indirect_list_newobj_is_none_with_writer(caplog):
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/906/906769.pdf"
-    name = "tika-906769.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+def test_sweep_recursion2_with_writer(url, name, pdf_file_path):
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     merger = PdfWriter()
     merger.append(reader)
-    merger.write("tmp-merger-do-not-commit.pdf")
+    merger.write(pdf_file_path)
+    merger.close()
+
+    reader2 = PdfReader(pdf_file_path)
+    reader2.pages
+
+
+@pytest.mark.enable_socket()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
+def test_sweep_indirect_list_newobj_is_none(caplog, pdf_file_path):
+    url = "https://corpora.tika.apache.org/base/docs/govdocs1/906/906769.pdf"
+    name = "tika-906769.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    merger = PdfMerger()
+    merger.append(reader)
+    merger.write(pdf_file_path)
     merger.close()
     # used to be: assert "Object 21 0 not defined." in caplog.text
 
-    reader2 = PdfReader("tmp-merger-do-not-commit.pdf")
+    reader2 = PdfReader(pdf_file_path)
     reader2.pages
 
-    # cleanup
-    os.remove("tmp-merger-do-not-commit.pdf")
+
+@pytest.mark.enable_socket()
+def test_sweep_indirect_list_newobj_is_none_with_writer(caplog, pdf_file_path):
+    url = "https://corpora.tika.apache.org/base/docs/govdocs1/906/906769.pdf"
+    name = "tika-906769.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    merger = PdfWriter()
+    merger.append(reader)
+    merger.write(pdf_file_path)
+    merger.close()
+    # used to be: assert "Object 21 0 not defined." in caplog.text
+
+    reader2 = PdfReader(pdf_file_path)
+    reader2.pages
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_iss1145():
     # issue with FitH destination with null param
     url = "https://github.com/py-pdf/pypdf/files/9164743/file-0.pdf"
     name = "iss1145.pdf"
     merger = PdfMerger()
-    merger.append(PdfReader(BytesIO(get_pdf_from_url(url, name=name))))
+    merger.append(PdfReader(BytesIO(get_data_from_url(url, name=name))))
     merger.close()
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_iss1145_with_writer():
     # issue with FitH destination with null param
     url = "https://github.com/py-pdf/pypdf/files/9164743/file-0.pdf"
     name = "iss1145.pdf"
     merger = PdfWriter()
-    merger.append(PdfReader(BytesIO(get_pdf_from_url(url, name=name))))
+    merger.append(PdfReader(BytesIO(get_data_from_url(url, name=name))))
     merger.close()
 
 
-def test_deprecation_bookmark_decorator_deprecationexcp():
-    reader = PdfReader(RESOURCE_ROOT / "outlines-with-invalid-destinations.pdf")
-    merger = PdfMerger()
-    with pytest.raises(
-        DeprecationError,
-        match=(
-            "import_bookmarks is deprecated as an argument. "
-            "Use import_outline instead"
-        ),
-    ):
-        merger.merge(0, reader, import_bookmarks=True)
-
-
-def test_deprecation_bookmark_decorator_deprecationexcp_with_writer():
-    reader = PdfReader(RESOURCE_ROOT / "outlines-with-invalid-destinations.pdf")
-    merger = PdfWriter()
-    with pytest.raises(
-        DeprecationError,
-        match=(
-            "import_bookmarks is deprecated as an argument. "
-            "Use import_outline instead"
-        ),
-    ):
-        merger.merge(0, reader, import_bookmarks=True)
-
-
-def test_deprecation_bookmark_decorator_output():
-    reader = PdfReader(RESOURCE_ROOT / "outlines-with-invalid-destinations.pdf")
-    merger = PdfMerger()
-    with pytest.raises(DeprecationError):
-        merger.merge(0, reader, import_bookmarks=True)
-
-
-def test_deprecation_bookmark_decorator_output_with_writer():
-    reader = PdfReader(RESOURCE_ROOT / "outlines-with-invalid-destinations.pdf")
-    merger = PdfWriter()
-    with pytest.raises(DeprecationError):
-        merger.merge(0, reader, import_bookmarks=True)
-
-
-@pytest.mark.external
+@pytest.mark.enable_socket()
+@pytest.mark.filterwarnings("ignore::DeprecationWarning")
 def test_iss1344(caplog):
     url = "https://github.com/py-pdf/pypdf/files/9549001/input.pdf"
     name = "iss1344.pdf"
     m = PdfMerger()
-    m.append(PdfReader(BytesIO(get_pdf_from_url(url, name=name))))
+    m.append(PdfReader(BytesIO(get_data_from_url(url, name=name))))
     b = BytesIO()
     m.write(b)
     r = PdfReader(b)
@@ -695,12 +616,12 @@ def test_iss1344(caplog):
     assert r.threads is None
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_iss1344_with_writer(caplog):
     url = "https://github.com/py-pdf/pypdf/files/9549001/input.pdf"
     name = "iss1344.pdf"
     m = PdfWriter()
-    m.append(PdfReader(BytesIO(get_pdf_from_url(url, name=name))))
+    m.append(PdfReader(BytesIO(get_data_from_url(url, name=name))))
     b = BytesIO()
     m.write(b)
     p = PdfReader(b).pages[0]
@@ -708,14 +629,19 @@ def test_iss1344_with_writer(caplog):
     assert "adresse où le malade peut être visité" in p.extract_text()
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_articles_with_writer(caplog):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924666.pdf"
     name = "924666.pdf"
     m = PdfWriter()
-    m.append(PdfReader(BytesIO(get_pdf_from_url(url, name=name))), (2, 10))
+    m.append(PdfReader(BytesIO(get_data_from_url(url, name=name))), (2, 10))
     b = BytesIO()
     m.write(b)
     r = PdfReader(b)
     assert len(r.threads) == 4
     assert r.threads[0].get_object()["/F"]["/P"] == r.pages[0]
+
+
+def test_deprecate_pdfmerger():
+    with pytest.warns(DeprecationWarning), PdfMerger() as merger:
+        merger.append(RESOURCE_ROOT / "crazyones.pdf")
