@@ -1,21 +1,22 @@
+"""Test the pypdf._reader module."""
 import io
-import os
 import time
 from io import BytesIO
 from pathlib import Path
+from typing import List, Union
 
 import pytest
 
-from pypdf import PdfReader
-from pypdf._reader import convert_to_int, convertToInt
+from pypdf import PdfReader, PdfWriter
+from pypdf._crypt_providers import crypt_provider
+from pypdf._reader import convert_to_int
 from pypdf.constants import ImageAttributes as IA
 from pypdf.constants import PageAttributes as PG
+from pypdf.constants import UserAccessPermissions as UAP
 from pypdf.errors import (
-    DeprecationError,
     EmptyFileError,
     FileNotDecryptedError,
     PdfReadError,
-    PdfReadWarning,
     WrongPasswordError,
 )
 from pypdf.generic import (
@@ -27,21 +28,18 @@ from pypdf.generic import (
     TextStringObject,
 )
 
-from . import get_pdf_from_url, normalize_warnings
-
-try:
-    from Crypto.Cipher import AES  # noqa: F401
-
-    HAS_PYCRYPTODOME = True
-except ImportError:
-    HAS_PYCRYPTODOME = False
+from . import get_data_from_url, normalize_warnings
 
+HAS_AES = crypt_provider[0] in ["pycryptodome", "cryptography"]
 TESTS_ROOT = Path(__file__).parent.resolve()
 PROJECT_ROOT = TESTS_ROOT.parent
 RESOURCE_ROOT = PROJECT_ROOT / "resources"
 SAMPLE_ROOT = PROJECT_ROOT / "sample-files"
 
 
+NestedList = Union[int, None, List["NestedList"]]
+
+
 @pytest.mark.parametrize(
     ("src", "num_pages"),
     [("selenium-pypdf-issue-177.pdf", 1), ("pdflatex-outline.pdf", 4)],
@@ -50,6 +48,8 @@ def test_get_num_pages(src, num_pages):
     src = RESOURCE_ROOT / src
     reader = PdfReader(src)
     assert len(reader.pages) == num_pages
+    # from #1911
+    assert "/Size" in reader.trailer
 
 
 @pytest.mark.parametrize(
@@ -110,7 +110,24 @@ def test_read_metadata(pdf_path, expected):
             assert metadict["/Title"] == docinfo.title
 
 
-@pytest.mark.samples
+def test_iss1943():
+    reader = PdfReader(RESOURCE_ROOT / "crazyones.pdf")
+    docinfo = reader.metadata
+    docinfo.update(
+        {
+            NameObject("/CreationDate"): TextStringObject("D:20230705005151Z00'00'"),
+            NameObject("/ModDate"): TextStringObject("D:20230705005151Z00'00'"),
+        }
+    )
+    docinfo.creation_date
+    docinfo.creation_date_raw
+    docinfo.modification_date
+    docinfo.modification_date_raw
+    docinfo.update({NameObject("/CreationDate"): NumberObject(1)})
+    assert docinfo.creation_date is None
+
+
+@pytest.mark.samples()
 @pytest.mark.parametrize(
     "pdf_path", [SAMPLE_ROOT / "017-unreadable-meta-data/unreadablemetadata.pdf"]
 )
@@ -179,7 +196,7 @@ def test_get_outline(src, outline_elements):
     assert len(outline) == outline_elements
 
 
-@pytest.mark.samples
+@pytest.mark.samples()
 @pytest.mark.parametrize(
     ("src", "expected_images"),
     [
@@ -226,7 +243,7 @@ def test_get_images(src, expected_images):
                 )
         finally:
             try:
-                os.remove(fn)
+                Path(fn).unlink()
             except Exception:
                 pass
 
@@ -317,7 +334,7 @@ def test_get_images_raw(
     )
     pdf_stream = io.BytesIO(pdf_data)
     if should_fail:
-        with pytest.raises(PdfReadError) as exc, pytest.warns(PdfReadWarning):
+        with pytest.raises(PdfReadError) as exc:
             PdfReader(pdf_stream, strict=strict)
         assert exc.type == PdfReadError
         if startx_correction == -1:
@@ -390,14 +407,14 @@ def test_get_page_of_encrypted_file(pdffile, password, should_fail):
         ),
     ],
 )
-def test_get_form(src, expected, expected_get_fields):
+def test_get_form(src, expected, expected_get_fields, txt_file_path):
     """Check if we can read out form data."""
     src = RESOURCE_ROOT / src
     reader = PdfReader(src)
     fields = reader.get_form_text_fields()
     assert fields == expected
 
-    with open("tmp-fields-report.txt", "w") as f:
+    with open(txt_file_path, "w") as f:
         fields = reader.get_fields(fileobj=f)
     assert fields == expected_get_fields
     if fields:
@@ -416,9 +433,6 @@ def test_get_form(src, expected, expected_get_fields):
                 field.additional_actions,
             ]
 
-    # cleanup
-    os.remove("tmp-fields-report.txt")
-
 
 @pytest.mark.parametrize(
     ("src", "page_number"),
@@ -463,10 +477,16 @@ def test_read_empty():
     assert exc.value.args[0] == "Cannot read an empty file"
 
 
-def test_read_malformed_header():
+def test_read_malformed_header(caplog):
     with pytest.raises(PdfReadError) as exc:
         PdfReader(io.BytesIO(b"foo"), strict=True)
     assert exc.value.args[0] == "PDF starts with 'foo', but '%PDF-' expected"
+    caplog.clear()
+    try:
+        PdfReader(io.BytesIO(b"foo"), strict=False)
+    except Exception:
+        pass
+    assert caplog.messages[0].startswith("invalid pdf header")
 
 
 def test_read_malformed_body():
@@ -509,7 +529,7 @@ def test_read_prev_0_trailer():
         pdf_data.find(b"xref") - 1,
     )
     pdf_stream = io.BytesIO(pdf_data)
-    with pytest.raises(PdfReadError) as exc, pytest.warns(PdfReadWarning):
+    with pytest.raises(PdfReadError) as exc:
         PdfReader(pdf_stream, strict=True)
     assert exc.value.args[0] == "/Prev=0 in the trailer (try opening with strict=False)"
 
@@ -586,7 +606,7 @@ def test_read_unknown_zero_pages(caplog):
         "Xref table not zero-indexed. ID numbers for objects will be corrected.",
     ]
     assert normalize_warnings(caplog.text) == warnings
-    with pytest.raises(PdfReadError) as exc, pytest.warns(PdfReadWarning):
+    with pytest.raises(PdfReadError) as exc:
         len(reader.pages)
 
     assert exc.value.args[0] == "Could not find object."
@@ -596,7 +616,7 @@ def test_read_unknown_zero_pages(caplog):
         "startxref on same line as offset",
     ]
     assert normalize_warnings(caplog.text) == warnings
-    with pytest.raises(AttributeError) as exc, pytest.warns(PdfReadWarning):
+    with pytest.raises(AttributeError) as exc:
         len(reader.pages)
     assert exc.value.args[0] == "'NoneType' object has no attribute 'get_object'"
 
@@ -619,8 +639,10 @@ def test_get_destination_page_number():
 
 
 def test_do_not_get_stuck_on_large_files_without_start_xref():
-    """Tests for the absence of a DoS bug, where a large file without an
-    startxref mark would cause the library to hang for minutes to hours."""
+    """
+    Tests for the absence of a DoS bug, where a large file without an
+    startxref mark would cause the library to hang for minutes to hours.
+    """
     start_time = time.time()
     broken_stream = BytesIO(b"\0" * 5 * 1000 * 1000)
     with pytest.raises(PdfReadError):
@@ -631,7 +653,7 @@ def test_do_not_get_stuck_on_large_files_without_start_xref():
     assert parse_duration < 60
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_decrypt_when_no_id():
     """
     Decrypt an encrypted file that's missing the 'ID' value in its trailer.
@@ -658,13 +680,13 @@ def test_reader_properties():
     [True, False],
 )
 def test_issue604(caplog, strict):
-    """Test with invalid destinations."""  # todo
+    """Test with invalid destinations."""  # TODO
     with open(RESOURCE_ROOT / "issue-604.pdf", "rb") as f:
         pdf = None
         outline = None
         if strict:
             pdf = PdfReader(f, strict=strict)
-            with pytest.raises(PdfReadError) as exc, pytest.warns(PdfReadWarning):
+            with pytest.raises(PdfReadError) as exc:
                 outline = pdf.outline
             if "Unknown Destination" not in exc.value.args[0]:
                 raise Exception("Expected exception not raised")
@@ -677,18 +699,20 @@ def test_issue604(caplog, strict):
             ]
             assert normalize_warnings(caplog.text) == msg
 
-        def get_dest_pages(x):
+        def get_dest_pages(x) -> NestedList:
             if isinstance(x, list):
-                r = [get_dest_pages(y) for y in x]
-                return r
+                return [get_dest_pages(y) for y in x]
             else:
-                return pdf.get_destination_page_number(x) + 1
+                destination_page_number = pdf.get_destination_page_number(x)
+                if destination_page_number is None:
+                    return destination_page_number
+                return destination_page_number + 1
 
         out = []
 
         # oi can be destination or a list:preferred to just print them
         for oi in outline:
-            out.append(get_dest_pages(oi))
+            out.append(get_dest_pages(oi))  # noqa: PERF401
 
 
 def test_decode_permissions():
@@ -706,11 +730,54 @@ def test_decode_permissions():
 
     print_ = base.copy()
     print_["print"] = True
-    assert reader.decode_permissions(4) == print_
+    with pytest.raises(
+        DeprecationWarning,
+        match="decode_permissions is deprecated and will be removed in pypdf 5.0.0. Use user_access_permissions instead",  # noqa: E501
+    ):
+        assert reader.decode_permissions(4) == print_
 
     modify = base.copy()
     modify["modify"] = True
-    assert reader.decode_permissions(8) == modify
+    with pytest.raises(
+        DeprecationWarning,
+        match="decode_permissions is deprecated and will be removed in pypdf 5.0.0. Use user_access_permissions instead",  # noqa: E501
+    ):
+        assert reader.decode_permissions(8) == modify
+
+
+@pytest.mark.skipif(not HAS_AES, reason="No AES implementation")
+def test_user_access_permissions():
+    # Not encrypted.
+    reader = PdfReader(RESOURCE_ROOT / "crazyones.pdf")
+    assert reader.user_access_permissions is None
+
+    # Encrypted.
+    reader = PdfReader(RESOURCE_ROOT / "encryption" / "r6-owner-password.pdf")
+    assert reader.user_access_permissions == UAP.all()
+
+    # Custom writer permissions.
+    writer = PdfWriter(clone_from=RESOURCE_ROOT / "crazyones.pdf")
+    writer.encrypt(
+        user_password="",
+        owner_password="abc",
+        permissions_flag=UAP.PRINT | UAP.FILL_FORM_FIELDS,
+    )
+    output = BytesIO()
+    writer.write(output)
+    reader = PdfReader(output)
+    assert reader.user_access_permissions == (UAP.PRINT | UAP.FILL_FORM_FIELDS)
+
+    # All writer permissions.
+    writer = PdfWriter(clone_from=RESOURCE_ROOT / "crazyones.pdf")
+    writer.encrypt(
+        user_password="",
+        owner_password="abc",
+        permissions_flag=UAP.all(),
+    )
+    output = BytesIO()
+    writer.write(output)
+    reader = PdfReader(output)
+    assert reader.user_access_permissions == UAP.all()
 
 
 def test_pages_attribute():
@@ -741,22 +808,10 @@ def test_convert_to_int_error():
     assert exc.value.args[0] == "invalid size in convert_to_int"
 
 
-def test_convertToInt_deprecated():
-    msg = (
-        "convertToInt is deprecated and was removed in pypdf 3.0.0. "
-        "Use convert_to_int instead."
-    )
-    with pytest.raises(
-        DeprecationError,
-        match=msg,
-    ):
-        assert convertToInt(b"\x01", 8) == 1
-
-
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_iss925():
     url = "https://github.com/py-pdf/pypdf/files/8796328/1.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name="iss925.pdf")))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name="iss925.pdf")))
 
     for page_sliced in reader.pages:
         page_object = page_sliced.get_object()
@@ -809,13 +864,13 @@ def test_read_not_binary_mode(caplog):
     assert normalize_warnings(caplog.text) == [msg]
 
 
-@pytest.mark.external
-@pytest.mark.skipif(not HAS_PYCRYPTODOME, reason="No pycryptodome")
+@pytest.mark.enable_socket()
+@pytest.mark.skipif(not HAS_AES, reason="No AES algorithm available")
 def test_read_form_416():
     url = (
         "https://www.fda.gov/downloads/AboutFDA/ReportsManualsForms/Forms/UCM074728.pdf"
     )
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name="issue_416.pdf")))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name="issue_416.pdf")))
     fields = reader.get_form_text_fields()
     assert len(fields) > 0
 
@@ -848,63 +903,65 @@ def test_form_topname_with_and_without_acroform(caplog):
     assert "have a non-expected parent" in caplog.text
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_extract_text_xref_issue_2(caplog):
     # pdf/0264cf510015b2a4b395a15cb23c001e.pdf
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/981/981961.pdf"
     msg = "incorrect startxref pointer(2)"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name="tika-981961.pdf")))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name="tika-981961.pdf")))
     for page in reader.pages:
         page.extract_text()
     assert normalize_warnings(caplog.text) == [msg]
 
 
-@pytest.mark.external
-@pytest.mark.slow
+@pytest.mark.enable_socket()
+@pytest.mark.slow()
 def test_extract_text_xref_issue_3(caplog):
     # pdf/0264cf510015b2a4b395a15cb23c001e.pdf
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/977/977774.pdf"
     msg = "incorrect startxref pointer(3)"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name="tika-977774.pdf")))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name="tika-977774.pdf")))
     for page in reader.pages:
         page.extract_text()
     assert normalize_warnings(caplog.text) == [msg]
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_extract_text_pdf15():
     # pdf/0264cf510015b2a4b395a15cb23c001e.pdf
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/976/976030.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name="tika-976030.pdf")))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name="tika-976030.pdf")))
     for page in reader.pages:
         page.extract_text()
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_extract_text_xref_table_21_bytes_clrf():
     # pdf/0264cf510015b2a4b395a15cb23c001e.pdf
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/956/956939.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name="tika-956939.pdf")))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name="tika-956939.pdf")))
     for page in reader.pages:
         page.extract_text()
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_get_fields():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/972/972486.pdf"
     name = "tika-972486.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     fields = reader.get_fields()
     assert fields is not None
     assert "c1-1" in fields
-    assert dict(fields["c1-1"]) == ({"/FT": "/Btn", "/T": "c1-1"})
+    assert dict(fields["c1-1"]) == (
+        {"/FT": "/Btn", "/T": "c1-1", "/_States_": ["/On", "/Off"]}
+    )
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_get_full_qualified_fields():
     url = "https://github.com/py-pdf/pypdf/files/10142389/fields_with_dots.pdf"
     name = "fields_with_dots.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     fields = reader.get_form_text_fields(True)
     assert fields is not None
     assert "customer.name" in fields
@@ -920,52 +977,49 @@ def test_get_full_qualified_fields():
     assert fields["customer.name"]["/T"] == "name"
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 @pytest.mark.filterwarnings("ignore::pypdf.errors.PdfReadWarning")
 def test_get_fields_read_else_block():
     # covers also issue 1089
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/934/934771.pdf"
     name = "tika-934771.pdf"
-    PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    PdfReader(BytesIO(get_data_from_url(url, name=name)))
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_get_fields_read_else_block2():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/914/914902.pdf"
     name = "tika-914902.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     fields = reader.get_fields()
     assert fields is None
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 @pytest.mark.filterwarnings("ignore::pypdf.errors.PdfReadWarning")
 def test_get_fields_read_else_block3():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/957/957721.pdf"
     name = "tika-957721.pdf"
-    PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    PdfReader(BytesIO(get_data_from_url(url, name=name)))
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_metadata_is_none():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/963/963692.pdf"
     name = "tika-963692.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     assert reader.metadata is None
 
 
-@pytest.mark.external
-def test_get_fields_read_write_report():
+@pytest.mark.enable_socket()
+def test_get_fields_read_write_report(txt_file_path):
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/909/909655.pdf"
     name = "tika-909655.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
-    with open("tmp-fields-report.txt", "w") as fp:
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    with open(txt_file_path, "w") as fp:
         fields = reader.get_fields(fileobj=fp)
     assert fields
 
-    # cleanup
-    os.remove("tmp-fields-report.txt")
-
 
 @pytest.mark.parametrize(
     "src",
@@ -979,11 +1033,11 @@ def test_xfa(src):
     assert reader.xfa is None
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_xfa_non_empty():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/942/942050.pdf"
     name = "tika-942050.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     assert list(reader.xfa.keys()) == [
         "preamble",
         "config",
@@ -995,7 +1049,7 @@ def test_xfa_non_empty():
 
 
 @pytest.mark.parametrize(
-    "src,pdf_header",
+    ("src", "pdf_header"),
     [
         (RESOURCE_ROOT / "attachment.pdf", "%PDF-1.5"),
         (RESOURCE_ROOT / "crazyones.pdf", "%PDF-1.5"),
@@ -1007,19 +1061,15 @@ def test_header(src, pdf_header):
     assert reader.pdf_header == pdf_header
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_outline_color():
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924546.pdf"
-    name = "tika-924546.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(name="tika-924546.pdf")))
     assert reader.outline[0].color == [0, 0, 1]
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_outline_font_format():
-    url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924546.pdf"
-    name = "tika-924546.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(name="tika-924546.pdf")))
     assert reader.outline[0].font_format == 2
 
 
@@ -1036,7 +1086,7 @@ def get_outline_property(outline, attribute_name: str):
     return results
 
 
-@pytest.mark.samples
+@pytest.mark.samples()
 def test_outline_title_issue_1121():
     reader = PdfReader(SAMPLE_ROOT / "014-outlines/mistitled_outlines_example.pdf")
 
@@ -1083,7 +1133,7 @@ def test_outline_title_issue_1121():
     ]
 
 
-@pytest.mark.samples
+@pytest.mark.samples()
 def test_outline_count():
     reader = PdfReader(SAMPLE_ROOT / "014-outlines/mistitled_outlines_example.pdf")
 
@@ -1142,39 +1192,43 @@ def test_outline_missing_title(caplog):
     assert reader.outline[0]["/Title"] == ""
 
 
-@pytest.mark.external
-def test_named_destination():
-    # 1st case : the named_dest are stored directly as a dictionnary, PDF1.1 style
-    url = "https://github.com/py-pdf/pypdf/files/9197028/lorem_ipsum.pdf"
-    name = "lorem_ipsum.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+@pytest.mark.enable_socket()
+@pytest.mark.parametrize(
+    ("url", "name"),
+    [
+        # 1st case : the named_dest are stored directly as a dictionnary, PDF1.1 style
+        (
+            "https://github.com/py-pdf/pypdf/files/9197028/lorem_ipsum.pdf",
+            "lorem_ipsum.pdf",
+        ),
+        # 2nd case : Dest below names and with Kids...
+        (
+            "https://github.com/py-pdf/pypdf/files/11714214/PDF32000_2008.pdf",
+            "PDF32000_2008.pdf",
+        )
+        # 3nd case : Dests with Name tree (TODO: Add this case)
+    ],
+    ids=["stored_directly", "dest_below_names_with_kids"],
+)
+def test_named_destination(url, name):
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     assert len(reader.named_destinations) > 0
-    # 2nd case : Dest below names and with Kids...
-    url = (
-        "https://opensource.adobe.com/dc-acrobat-sdk-docs/standards/"
-        "pdfstandards/pdf/PDF32000_2008.pdf"
-    )
-    name = "PDF32000_2008.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
-    assert len(reader.named_destinations) > 0
-    # 3nd case : Dests with Name tree
-    # TODO : case to be added
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_outline_with_missing_named_destination():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/913/913678.pdf"
     name = "tika-913678.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     # outline items in document reference a named destination that is not defined
     assert reader.outline[1][0].title.startswith("Report for 2002AZ3B: Microbial")
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_outline_with_empty_action():
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924546.pdf"
     name = "tika-924546.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     # outline items (entitled Tables and Figures) utilize an empty action (/A)
     # that has no type or destination
     assert reader.outline[-4].title == "Tables"
@@ -1187,12 +1241,12 @@ def test_outline_with_invalid_destinations():
     assert len(reader.outline) == 9
 
 
-@pytest.mark.external
-def test_PdfReaderMultipleDefinitions(caplog):
-    # iss325
+@pytest.mark.enable_socket()
+def test_pdfreader_multiple_definitions(caplog):
+    """iss325"""
     url = "https://github.com/py-pdf/pypdf/files/9176644/multipledefs.pdf"
     name = "multipledefs.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     reader.pages[0].extract_text()
     assert normalize_warnings(caplog.text) == [
         "Multiple definitions in dictionary at byte 0xb5 for key /Group"
@@ -1213,25 +1267,25 @@ def test_get_page_number_by_indirect():
     reader._get_page_number_by_indirect(1)
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_corrupted_xref_table():
     # issue #1292
     url = "https://github.com/py-pdf/pypdf/files/9444747/BreezeManual.orig.pdf"
     name = "BreezeMan1.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     reader.pages[0].extract_text()
     url = "https://github.com/py-pdf/pypdf/files/9444748/BreezeManual.failed.pdf"
     name = "BreezeMan2.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     reader.pages[0].extract_text()
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_reader(caplog):
     # iss #1273
     url = "https://github.com/py-pdf/pypdf/files/9464742/shiv_resume.pdf"
     name = "shiv_resume.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     assert "Previous trailer can not be read" in caplog.text
     caplog.clear()
     # first call requires some reparations...
@@ -1244,7 +1298,7 @@ def test_reader(caplog):
     assert caplog.text == ""
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_zeroing_xref():
     # iss #328
     url = (
@@ -1252,31 +1306,31 @@ def test_zeroing_xref():
         "UTA_OSHA_3115_Fall_Protection_Training_09162021_.pdf"
     )
     name = "UTA_OSHA.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     len(reader.pages)
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_thread():
     url = (
         "https://github.com/py-pdf/pypdf/files/9066120/"
         "UTA_OSHA_3115_Fall_Protection_Training_09162021_.pdf"
     )
     name = "UTA_OSHA.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     assert reader.threads is None
     url = "https://corpora.tika.apache.org/base/docs/govdocs1/924/924666.pdf"
     name = "tika-924666.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     assert isinstance(reader.threads, ArrayObject)
     assert len(reader.threads) >= 1
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_build_outline_item(caplog):
     url = "https://github.com/py-pdf/pypdf/files/9464742/shiv_resume.pdf"
     name = "shiv_resume.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     outline = reader._build_outline_item(
         DictionaryObject(
             {
@@ -1300,7 +1354,7 @@ def test_build_outline_item(caplog):
     assert "Unexpected destination 2" in exc.value.args[0]
 
 
-@pytest.mark.samples
+@pytest.mark.samples()
 @pytest.mark.parametrize(
     ("src", "page_labels"),
     [
@@ -1324,10 +1378,123 @@ def test_page_labels(src, page_labels):
     assert PdfReader(src).page_labels[:max_indices] == page_labels[:max_indices]
 
 
-@pytest.mark.external
+@pytest.mark.enable_socket()
 def test_iss1559():
     url = "https://github.com/py-pdf/pypdf/files/10441992/default.pdf"
     name = "iss1559.pdf"
-    reader = PdfReader(BytesIO(get_pdf_from_url(url, name=name)))
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     for p in reader.pages:
         p.extract_text()
+
+
+@pytest.mark.enable_socket()
+def test_iss1652():
+    # test of an annotation(link) directly stored in the /Annots in the page
+    url = "https://github.com/py-pdf/pypdf/files/10818844/tt.pdf"
+    name = "invalidNamesDest.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    reader.named_destinations
+
+
+@pytest.mark.enable_socket()
+def test_iss1689():
+    url = "https://github.com/py-pdf/pypdf/files/10948283/error_file_without_data.pdf"
+    name = "iss1689.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    reader.pages[0]
+
+
+@pytest.mark.enable_socket()
+def test_iss1710():
+    url = "https://nlp.stanford.edu/IR-book/pdf/irbookonlinereading.pdf"
+    name = "irbookonlinereading.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    reader.outline
+
+
+def test_broken_file_header():
+    pdf_data = (
+        b"%%PDF-\xa0sd\n"
+        b"1 0 obj << /Count 1 /Kids [4 0 R] /Type /Pages >> endobj\n"
+        b"2 0 obj << >> endobj\n"
+        b"3 0 obj << >> endobj\n"
+        b"4 0 obj << /Contents 3 0 R /CropBox [0.0 0.0 2550.0 3508.0]"
+        b" /MediaBox [0.0 0.0 2550.0 3508.0] /Parent 1 0 R"
+        b" /Resources << /Font << >> >>"
+        b" /Rotate 0 /Type /Page >> endobj\n"
+        b"5 0 obj << /Pages 1 0 R /Type /Catalog >> endobj\n"
+        b"xref 1 5\n"
+        b"%010d 00000 n\n"
+        b"%010d 00000 n\n"
+        b"%010d 00000 n\n"
+        b"%010d 00000 n\n"
+        b"%010d 00000 n\n"
+        b"trailer << %s/Root 5 0 R /Size 6 >>\n"
+        b"startxref %d\n"
+        b"%%%%EOF"
+    )
+    with_prev_0 = True
+    pdf_data = pdf_data % (
+        pdf_data.find(b"1 0 obj"),
+        pdf_data.find(b"2 0 obj"),
+        pdf_data.find(b"3 0 obj"),
+        pdf_data.find(b"4 0 obj"),
+        pdf_data.find(b"5 0 obj"),
+        b"/Prev 0 " if with_prev_0 else b"",
+        pdf_data.find(b"xref") - 1,
+    )
+    PdfReader(io.BytesIO(pdf_data))
+
+
+@pytest.mark.enable_socket()
+def test_iss1756():
+    url = "https://github.com/py-pdf/pypdf/files/11105591/641-Attachment-B-Pediatric-Cardiac-Arrest-8-1-2019.pdf"
+    name = "iss1756.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    reader.trailer["/ID"]
+    # removed to cope with missing cryptodome during commit check : len(reader.pages)
+
+
+@pytest.mark.enable_socket()
+@pytest.mark.timeout(30)
+def test_iss1825():
+    url = "https://github.com/py-pdf/pypdf/files/11367871/MiFO_LFO_FEIS_NOA_Published.3.pdf"
+    name = "iss1825.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    page = reader.pages[0]
+    page.extract_text()
+
+
+@pytest.mark.enable_socket()
+def test_iss2082():
+    url = "https://github.com/py-pdf/pypdf/files/12317939/test.pdf"
+    name = "iss2082.pdf"
+    b = get_data_from_url(url, name=name)
+    reader = PdfReader(BytesIO(b))
+    reader.pages[0].extract_text()
+
+    bb = bytearray(b)
+    bb[b.find(b"xref") + 2] = ord(b"E")
+    with pytest.raises(PdfReadError):
+        reader = PdfReader(BytesIO(bb))
+
+
+@pytest.mark.enable_socket()
+def test_issue_140():
+    url = "https://github.com/py-pdf/pypdf/files/12168578/bad_pdf_example.pdf"
+    name = "issue-140.pdf"
+    b = get_data_from_url(url, name=name)
+    reader = PdfReader(BytesIO(b))
+    assert len(reader.pages) == 54
+
+
+@pytest.mark.enable_socket()
+def test_xyz_with_missing_param():
+    """Cf #2236"""
+    url = "https://github.com/py-pdf/pypdf/files/12795356/tt1.pdf"
+    name = "issue2236.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+    assert reader.outline[0]["/Left"] == 820
+    assert reader.outline[0]["/Top"] == 0
+    assert reader.outline[1]["/Left"] == 0
+    assert reader.outline[0]["/Top"] == 0
